@@ -19,6 +19,7 @@ package org.apache.ratis.grpc;
 
 import org.apache.ratis.conf.Parameters;
 import org.apache.ratis.conf.RaftProperties;
+import org.apache.ratis.grpc.server.GrpcServices;
 import org.apache.ratis.server.RaftServerConfigKeys;
 import org.apache.ratis.util.SizeInBytes;
 import org.apache.ratis.util.TimeDuration;
@@ -53,50 +54,29 @@ public interface GrpcConfigKeys {
   interface TLS {
     String PREFIX = GrpcConfigKeys.PREFIX + ".tls";
 
-    String ENABLED_KEY = PREFIX + ".enabled";
-    boolean ENABLED_DEFAULT = false;
-    static boolean enabled(RaftProperties properties) {
-      return getBoolean(properties::getBoolean, ENABLED_KEY, ENABLED_DEFAULT, getDefaultLog());
-    }
+    @Deprecated
     static void setEnabled(RaftProperties properties, boolean enabled) {
-      setBoolean(properties::setBoolean, ENABLED_KEY, enabled);
+      LOG.warn("This method has no effect. Use setConf(Parameters, GrpcTlsConfig) instead.");
     }
 
-    String MUTUAL_AUTHN_ENABLED_KEY = PREFIX + ".mutual_authn.enabled";
-    boolean MUTUAL_AUTHN_ENABLED_DEFAULT = false;
-    static boolean mutualAuthnEnabled(RaftProperties properties) {
-      return getBoolean(properties::getBoolean,
-          MUTUAL_AUTHN_ENABLED_KEY, MUTUAL_AUTHN_ENABLED_DEFAULT, getDefaultLog());
-    }
+    @Deprecated
     static void setMutualAuthnEnabled(RaftProperties properties, boolean mutualAuthnEnabled) {
-      setBoolean(properties::setBoolean, MUTUAL_AUTHN_ENABLED_KEY, mutualAuthnEnabled);
+      LOG.warn("This method has no effect. Use setConf(Parameters, GrpcTlsConfig) instead.");
     }
 
-    String PRIVATE_KEY_FILE_NAME_KEY = PREFIX + ".private.key.file.name";
-    String PRIVATE_KEY_FILE_NAME_DEFAULT = "private.pem";
-    static String privateKeyFileName(RaftProperties properties) {
-      return get(properties::get, PRIVATE_KEY_FILE_NAME_KEY, PRIVATE_KEY_FILE_NAME_DEFAULT, getDefaultLog());
-    }
+    @Deprecated
     static void setPrivateKeyFileName(RaftProperties properties, String privateKeyFileName) {
-      set(properties::set, PRIVATE_KEY_FILE_NAME_KEY, privateKeyFileName);
+      LOG.warn("This method has no effect. Use setConf(Parameters, GrpcTlsConfig) instead.");
     }
 
-    String CERT_CHAIN_FILE_NAME_KEY = PREFIX + ".cert.chain.file.name";
-    String CERT_CHAIN_FILE_NAME_DEFAULT = "certificate.crt";
-    static String certChainFileName(RaftProperties properties) {
-      return get(properties::get, CERT_CHAIN_FILE_NAME_KEY, CERT_CHAIN_FILE_NAME_DEFAULT, getDefaultLog());
-    }
+    @Deprecated
     static void setCertChainFileName(RaftProperties properties, String certChainFileName) {
-      set(properties::set, CERT_CHAIN_FILE_NAME_KEY, certChainFileName);
+      LOG.warn("This method has no effect. Use setConf(Parameters, GrpcTlsConfig) instead.");
     }
 
-    String TRUST_STORE_KEY = PREFIX + ".trust.store";
-    String TRUST_STORE_DEFAULT = "ca.crt";
-    static String trustStore(RaftProperties properties) {
-      return get(properties::get, TRUST_STORE_KEY, TRUST_STORE_DEFAULT, getDefaultLog());
-    }
+    @Deprecated
     static void setTrustStore(RaftProperties properties, String trustStore) {
-      set(properties::set, TRUST_STORE_KEY, trustStore);
+      LOG.warn("This method has no effect. Use setConf(Parameters, GrpcTlsConfig) instead.");
     }
 
     String CONF_PARAMETER = PREFIX + ".conf";
@@ -230,15 +210,6 @@ public interface GrpcConfigKeys {
       setInt(properties::setInt, ASYNC_REQUEST_THREAD_POOL_SIZE_KEY, port);
     }
 
-    String TLS_CONF_PARAMETER = PREFIX + ".tls.conf";
-    Class<GrpcTlsConfig> TLS_CONF_CLASS = TLS.CONF_CLASS;
-    static GrpcTlsConfig tlsConf(Parameters parameters) {
-      return parameters != null ? parameters.get(TLS_CONF_PARAMETER, TLS_CONF_CLASS): null;
-    }
-    static void setTlsConf(Parameters parameters, GrpcTlsConfig conf) {
-      parameters.put(TLS_CONF_PARAMETER, conf, TLS_CONF_CLASS);
-    }
-
     String LEADER_OUTSTANDING_APPENDS_MAX_KEY = PREFIX + ".leader.outstanding.appends.max";
     int LEADER_OUTSTANDING_APPENDS_MAX_DEFAULT = 8;
     static int leaderOutstandingAppendsMax(RaftProperties properties) {
@@ -292,6 +263,25 @@ public interface GrpcConfigKeys {
       setTimeDuration(properties::setTimeDuration,
           LOG_MESSAGE_BATCH_DURATION_KEY, logMessageBatchDuration);
     }
+
+    String SERVICES_CUSTOMIZER_PARAMETER = PREFIX + ".services.customizer";
+    Class<GrpcServices.Customizer> SERVICES_CUSTOMIZER_CLASS = GrpcServices.Customizer.class;
+    static GrpcServices.Customizer servicesCustomizer(Parameters parameters) {
+      return parameters == null ? null
+          : parameters.get(SERVICES_CUSTOMIZER_PARAMETER, SERVICES_CUSTOMIZER_CLASS);
+    }
+    static void setServicesCustomizer(Parameters parameters, GrpcServices.Customizer customizer) {
+      parameters.put(SERVICES_CUSTOMIZER_PARAMETER, customizer, SERVICES_CUSTOMIZER_CLASS);
+    }
+
+    String TLS_CONF_PARAMETER = PREFIX + ".tls.conf";
+    Class<GrpcTlsConfig> TLS_CONF_CLASS = TLS.CONF_CLASS;
+    static GrpcTlsConfig tlsConf(Parameters parameters) {
+      return parameters != null ? parameters.get(TLS_CONF_PARAMETER, TLS_CONF_CLASS): null;
+    }
+    static void setTlsConf(Parameters parameters, GrpcTlsConfig conf) {
+      parameters.put(TLS_CONF_PARAMETER, conf, TLS_CONF_CLASS);
+    }
   }
 
   String MESSAGE_SIZE_MAX_KEY = PREFIX + ".message.size.max";
