@@ -32,11 +32,15 @@ import org.apache.ratis.server.ServerFactory;
 import org.apache.ratis.server.leader.FollowerInfo;
 import org.apache.ratis.server.leader.LeaderState;
 import org.apache.ratis.thirdparty.io.netty.buffer.PooledByteBufAllocator;
+import org.apache.ratis.thirdparty.io.netty.handler.ssl.SslContext;
 import org.apache.ratis.util.JavaUtils;
+import org.apache.ratis.util.MemoizedSupplier;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import java.util.function.BiFunction;
 import java.util.function.Consumer;
+import java.util.function.Supplier;
 
 public class GrpcFactory implements ServerFactory, ClientFactory {
 
@@ -65,18 +69,31 @@ public class GrpcFactory implements ServerFactory, ClientFactory {
     return value;
   }
 
+  static final BiFunction<GrpcTlsConfig, SslContext, SslContext> BUILD_SSL_CONTEXT_FOR_SERVER
+      = (tlsConf, defaultContext) -> tlsConf == null ? defaultContext : GrpcUtil.buildSslContextForServer(tlsConf);
+
+  static final BiFunction<GrpcTlsConfig, SslContext, SslContext> BUILD_SSL_CONTEXT_FOR_CLIENT
+      = (tlsConf, defaultContext) -> tlsConf == null ? defaultContext : GrpcUtil.buildSslContextForClient(tlsConf);
+
+  static final class SslContexts {
+    private final SslContext adminSslContext;
+    private final SslContext clientSslContext;
+    private final SslContext serverSslContext;
+
+    private SslContexts(GrpcTlsConfig tlsConfig, GrpcTlsConfig adminTlsConfig,
+        GrpcTlsConfig clientTlsConfig, GrpcTlsConfig serverTlsConfig,
+        BiFunction<GrpcTlsConfig, SslContext, SslContext> buildMethod) {
+      final SslContext defaultSslContext = buildMethod.apply(tlsConfig, null);
+      this.adminSslContext = buildMethod.apply(adminTlsConfig, defaultSslContext);
+      this.clientSslContext = buildMethod.apply(clientTlsConfig, defaultSslContext);
+      this.serverSslContext = buildMethod.apply(serverTlsConfig, defaultSslContext);
+    }
+  }
+
   private final GrpcServices.Customizer servicesCustomizer;
 
-  private final GrpcTlsConfig tlsConfig;
-  private final GrpcTlsConfig adminTlsConfig;
-  private final GrpcTlsConfig clientTlsConfig;
-  private final GrpcTlsConfig serverTlsConfig;
-
-  public static Parameters newRaftParameters(GrpcTlsConfig conf) {
-    final Parameters p = new Parameters();
-    GrpcConfigKeys.TLS.setConf(p, conf);
-    return p;
-  }
+  private final Supplier<SslContexts> forServerSupplier;
+  private final Supplier<SslContexts> forClientSupplier;
 
   public GrpcFactory(Parameters parameters) {
     this(GrpcConfigKeys.Server.servicesCustomizer(parameters),
@@ -87,35 +104,15 @@ public class GrpcFactory implements ServerFactory, ClientFactory {
     );
   }
 
-  public GrpcFactory(GrpcTlsConfig tlsConfig) {
-    this(null, tlsConfig, null, null, null);
-  }
-
   private GrpcFactory(GrpcServices.Customizer servicesCustomizer,
       GrpcTlsConfig tlsConfig, GrpcTlsConfig adminTlsConfig,
       GrpcTlsConfig clientTlsConfig, GrpcTlsConfig serverTlsConfig) {
     this.servicesCustomizer = servicesCustomizer;
 
-    this.tlsConfig = tlsConfig;
-    this.adminTlsConfig = adminTlsConfig;
-    this.clientTlsConfig = clientTlsConfig;
-    this.serverTlsConfig = serverTlsConfig;
-  }
-
-  public GrpcTlsConfig getTlsConfig() {
-    return tlsConfig;
-  }
-
-  public GrpcTlsConfig getAdminTlsConfig() {
-    return adminTlsConfig != null ? adminTlsConfig : tlsConfig;
-  }
-
-  public GrpcTlsConfig getClientTlsConfig() {
-    return clientTlsConfig != null ? clientTlsConfig : tlsConfig;
-  }
-
-  public GrpcTlsConfig getServerTlsConfig() {
-    return serverTlsConfig != null ? serverTlsConfig : tlsConfig;
+    this.forServerSupplier = MemoizedSupplier.valueOf(() -> new SslContexts(
+        tlsConfig, adminTlsConfig, clientTlsConfig, serverTlsConfig, BUILD_SSL_CONTEXT_FOR_SERVER));
+    this.forClientSupplier = MemoizedSupplier.valueOf(() -> new SslContexts(
+        tlsConfig, adminTlsConfig, clientTlsConfig, serverTlsConfig, BUILD_SSL_CONTEXT_FOR_CLIENT));
   }
 
   @Override
@@ -131,19 +128,24 @@ public class GrpcFactory implements ServerFactory, ClientFactory {
   @Override
   public GrpcServices newRaftServerRpc(RaftServer server) {
     checkPooledByteBufAllocatorUseCacheForAllThreads(LOG::info);
+
+    final SslContexts forServer = forServerSupplier.get();
+    final SslContexts forClient = forClientSupplier.get();
     return GrpcServicesImpl.newBuilder()
         .setServer(server)
         .setCustomizer(servicesCustomizer)
-        .setAdminTlsConfig(getAdminTlsConfig())
-        .setServerTlsConfig(getServerTlsConfig())
-        .setClientTlsConfig(getClientTlsConfig())
+        .setAdminSslContext(forServer.adminSslContext)
+        .setServerSslContextForServer(forServer.serverSslContext)
+        .setServerSslContextForClient(forClient.serverSslContext)
+        .setClientSslContext(forServer.clientSslContext)
         .build();
   }
 
   @Override
   public GrpcClientRpc newRaftClientRpc(ClientId clientId, RaftProperties properties) {
     checkPooledByteBufAllocatorUseCacheForAllThreads(LOG::debug);
-    return new GrpcClientRpc(clientId, properties,
-        getAdminTlsConfig(), getClientTlsConfig());
+
+    final SslContexts forClient = forClientSupplier.get();
+    return new GrpcClientRpc(clientId, properties, forClient.adminSslContext, forClient.clientSslContext);
   }
 }
