@@ -28,7 +28,10 @@ import org.apache.ratis.thirdparty.io.grpc.ManagedChannel;
 import org.apache.ratis.thirdparty.io.grpc.Metadata;
 import org.apache.ratis.thirdparty.io.grpc.Status;
 import org.apache.ratis.thirdparty.io.grpc.StatusRuntimeException;
+import org.apache.ratis.thirdparty.io.grpc.netty.GrpcSslContexts;
 import org.apache.ratis.thirdparty.io.grpc.stub.StreamObserver;
+import org.apache.ratis.thirdparty.io.netty.handler.ssl.ClientAuth;
+import org.apache.ratis.thirdparty.io.netty.handler.ssl.SslContext;
 import org.apache.ratis.thirdparty.io.netty.handler.ssl.SslContextBuilder;
 import org.apache.ratis.util.IOUtils;
 import org.apache.ratis.util.JavaUtils;
@@ -39,6 +42,7 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import javax.net.ssl.KeyManager;
+import javax.net.ssl.SSLException;
 import javax.net.ssl.TrustManager;
 import java.io.IOException;
 import java.util.concurrent.CompletableFuture;
@@ -46,6 +50,8 @@ import java.util.concurrent.TimeUnit;
 import java.util.function.Function;
 import java.util.function.Supplier;
 
+import static org.apache.ratis.thirdparty.io.netty.handler.ssl.SslProvider.OPENSSL;
+
 public interface GrpcUtil {
   Logger LOG = LoggerFactory.getLogger(GrpcUtil.class);
 
@@ -138,8 +144,10 @@ public interface GrpcUtil {
   static long getCallId(Throwable t) {
     if (t instanceof StatusRuntimeException) {
       final Metadata trailers = ((StatusRuntimeException)t).getTrailers();
-      String callId = trailers.get(CALL_ID);
-      return callId != null ? Integer.parseInt(callId) : -1;
+      if (trailers != null) {
+        final String callId = trailers.get(CALL_ID);
+        return callId != null ? Integer.parseInt(callId) : -1;
+      }
     }
     return -1;
   }
@@ -147,8 +155,8 @@ public interface GrpcUtil {
   static boolean isHeartbeat(Throwable t) {
     if (t instanceof StatusRuntimeException) {
       final Metadata trailers = ((StatusRuntimeException)t).getTrailers();
-      String isHeartbeat = trailers != null ? trailers.get(HEARTBEAT) : null;
-      return isHeartbeat != null && Boolean.valueOf(isHeartbeat);
+      final String isHeartbeat = trailers != null ? trailers.get(HEARTBEAT) : null;
+      return Boolean.parseBoolean(isHeartbeat);
     }
     return false;
   }
@@ -156,7 +164,7 @@ public interface GrpcUtil {
   static IOException unwrapIOException(Throwable t) {
     final IOException e;
     if (t instanceof StatusRuntimeException) {
-      e = GrpcUtil.unwrapException((StatusRuntimeException) t);
+      e = unwrapException((StatusRuntimeException) t);
     } else {
       e = IOUtils.asIOException(t);
     }
@@ -172,7 +180,7 @@ public interface GrpcUtil {
       supplier.get().whenComplete((reply, exception) -> {
         if (exception != null) {
           warning.accept(exception);
-          responseObserver.onError(GrpcUtil.wrapException(exception));
+          responseObserver.onError(wrapException(exception));
         } else {
           responseObserver.onNext(toProto.apply(reply));
           responseObserver.onCompleted();
@@ -180,7 +188,7 @@ public interface GrpcUtil {
       });
     } catch (Exception e) {
       warning.accept(e);
-      responseObserver.onError(GrpcUtil.wrapException(e));
+      responseObserver.onError(wrapException(e));
     }
   }
 
@@ -189,7 +197,7 @@ public interface GrpcUtil {
   }
 
   class StatusRuntimeExceptionMetadataBuilder {
-    private Metadata trailers = new Metadata();
+    private final Metadata trailers = new Metadata();
 
     StatusRuntimeExceptionMetadataBuilder(Throwable t) {
       trailers.put(EXCEPTION_TYPE_KEY, t.getClass().getCanonicalName());
@@ -297,4 +305,38 @@ public interface GrpcUtil {
       b.keyManager(privateKey.get(), certificates.get());
     }
   }
+
+  static SslContext buildSslContextForServer(GrpcTlsConfig tlsConf) {
+    if (tlsConf == null) {
+      return null;
+    }
+    SslContextBuilder b = initSslContextBuilderForServer(tlsConf.getKeyManager());
+    if (tlsConf.getMtlsEnabled()) {
+      b.clientAuth(ClientAuth.REQUIRE);
+      setTrustManager(b, tlsConf.getTrustManager());
+    }
+    b = GrpcSslContexts.configure(b, OPENSSL);
+    try {
+      return b.build();
+    } catch (Exception e) {
+      throw new IllegalArgumentException("Failed to buildSslContextForServer from tlsConfig " + tlsConf, e);
+    }
+  }
+
+  static SslContext buildSslContextForClient(GrpcTlsConfig tlsConf) {
+    if (tlsConf == null) {
+      return null;
+    }
+
+    final SslContextBuilder b = GrpcSslContexts.forClient();
+    setTrustManager(b, tlsConf.getTrustManager());
+    if (tlsConf.getMtlsEnabled()) {
+      setKeyManager(b, tlsConf.getKeyManager());
+    }
+    try {
+      return b.build();
+    } catch (SSLException e) {
+      throw new IllegalArgumentException("Failed to buildSslContextForClient from tlsConfig " + tlsConf, e);
+    }
+  }
 }
