@@ -83,7 +83,7 @@ import org.apache.ratis.server.impl.LeaderElection.Phase;
 import org.apache.ratis.server.impl.RetryCacheImpl.CacheEntry;
 import org.apache.ratis.server.impl.ServerImplUtils.ConsecutiveIndices;
 import org.apache.ratis.server.impl.ServerImplUtils.NavigableIndices;
-import org.apache.ratis.server.leader.LeaderState;
+import org.apache.ratis.server.leader.LeaderState.StepDownReason;
 import org.apache.ratis.server.metrics.LeaderElectionMetrics;
 import org.apache.ratis.server.metrics.RaftServerMetricsImpl;
 import org.apache.ratis.server.protocol.RaftServerAsynchronousProtocol;
@@ -139,6 +139,7 @@ import java.util.function.Supplier;
 import java.util.stream.Collectors;
 import java.util.stream.Stream;
 
+import static org.apache.ratis.server.impl.LeaderElection.Result.NOT_IN_CONF;
 import static org.apache.ratis.server.impl.ServerImplUtils.assertEntries;
 import static org.apache.ratis.server.impl.ServerImplUtils.assertGroup;
 import static org.apache.ratis.server.impl.ServerImplUtils.effectiveCommitIndex;
@@ -171,7 +172,7 @@ class RaftServerImpl implements RaftServer.Division,
 
     @Override
     public boolean isLeaderReady() {
-      return isLeader() && getRole().isLeaderReady();
+      return getRole().isLeaderReady();
     }
 
     @Override
@@ -409,7 +410,7 @@ class RaftServerImpl implements RaftServer.Division,
       startAsPeer(RaftPeerRole.LISTENER);
     } else {
       LOG.info("{}: start with initializing state, conf={}", getMemberId(), conf);
-      setRole(RaftPeerRole.FOLLOWER, "start");
+      setRole(RaftPeerRole.FOLLOWER, NOT_IN_CONF);
     }
 
     jmxAdapter.registerMBean();
@@ -560,12 +561,12 @@ class RaftServerImpl implements RaftServer.Division,
       try {
         ConcurrentUtils.shutdownAndWait(clientExecutor);
       } catch (Exception e) {
-        LOG.warn(getMemberId() + ": Failed to shutdown clientExecutor", e);
+        LOG.warn("{}: Failed to shutdown clientExecutor", getMemberId(), e);
       }
       try {
         ConcurrentUtils.shutdownAndWait(serverExecutor);
       } catch (Exception e) {
-        LOG.warn(getMemberId() + ": Failed to shutdown serverExecutor", e);
+        LOG.warn("{}: Failed to shutdown serverExecutor", getMemberId(), e);
       }
       closeFinishedLatch.countDown();
     });
@@ -752,6 +753,11 @@ class RaftServerImpl implements RaftServer.Division,
   }
 
   private CompletableFuture<RaftClientReply> checkLeaderState(RaftClientRequest request) {
+    try {
+      assertGroup(getMemberId(), request);
+    } catch (GroupMismatchException e) {
+      return JavaUtils.completeExceptionally(e);
+    }
     return checkLeaderState(request, null);
   }
 
@@ -759,12 +765,6 @@ class RaftServerImpl implements RaftServer.Division,
    * @return null if the server is in leader state.
    */
   private CompletableFuture<RaftClientReply> checkLeaderState(RaftClientRequest request, CacheEntry entry) {
-    try {
-      assertGroup(getMemberId(), request);
-    } catch (GroupMismatchException e) {
-      return RetryCacheImpl.failWithException(e, entry);
-    }
-
     if (!getInfo().isLeader()) {
       NotLeaderException exception = generateNotLeaderException();
       final RaftClientReply reply = newExceptionReply(request, exception);
@@ -809,6 +809,11 @@ class RaftServerImpl implements RaftServer.Division,
         getMemberId() + " is not in " + expected + ": current state is " + c), expected);
   }
 
+  private CompletableFuture<RaftClientReply> getResourceUnavailableReply(RaftClientRequest request, CacheEntry entry) {
+    return entry.failWithException(new ResourceUnavailableException(
+        getMemberId() + ": Failed to acquire a pending write request for " + request));
+    }
+
   /**
    * Handle a normal update request from client.
    */
@@ -819,6 +824,17 @@ class RaftServerImpl implements RaftServer.Division,
 
     assertLifeCycleState(LifeCycle.States.RUNNING);
 
+    final LeaderStateImpl unsyncedLeaderState = role.getLeaderState().orElse(null);
+    if (unsyncedLeaderState == null) {
+      final RaftClientReply reply = newExceptionReply(request, generateNotLeaderException());
+      return RetryCacheImpl.failWithReply(reply, cacheEntry);
+    }
+    final PendingRequests.Permit unsyncedPermit = unsyncedLeaderState.tryAcquirePendingRequest(request.getMessage());
+    if (unsyncedPermit == null) {
+      return getResourceUnavailableReply(request, cacheEntry);
+    }
+
+    final LeaderStateImpl leaderState;
     final PendingRequest pending;
     synchronized (this) {
       final CompletableFuture<RaftClientReply> reply = checkLeaderState(request, cacheEntry);
@@ -826,16 +842,15 @@ class RaftServerImpl implements RaftServer.Division,
         return reply;
       }
 
-      // append the message to its local log
-      final LeaderStateImpl leaderState = role.getLeaderStateNonNull();
-      writeIndexCache.add(request.getClientId(), context.getLogIndexFuture());
-
-      final PendingRequests.Permit permit = leaderState.tryAcquirePendingRequest(request.getMessage());
+      leaderState = role.getLeaderStateNonNull();
+      final PendingRequests.Permit permit = leaderState == unsyncedLeaderState? unsyncedPermit
+          : leaderState.tryAcquirePendingRequest(request.getMessage());
       if (permit == null) {
-        cacheEntry.failWithException(new ResourceUnavailableException(
-            getMemberId() + ": Failed to acquire a pending write request for " + request));
-        return cacheEntry.getReplyFuture();
+        return getResourceUnavailableReply(request, cacheEntry);
       }
+
+      // append the message to its local log
+      writeIndexCache.add(request.getClientId(), context.getLogIndexFuture());
       try {
         state.appendLog(context);
       } catch (StateMachineException e) {
@@ -845,7 +860,7 @@ class RaftServerImpl implements RaftServer.Division,
         cacheEntry.failWithReply(exceptionReply);
         // leader will step down here
         if (e.leaderShouldStepDown() && getInfo().isLeader()) {
-          leaderState.submitStepDownEvent(LeaderState.StepDownReason.STATE_MACHINE_EXCEPTION);
+          leaderState.submitStepDownEvent(StepDownReason.STATE_MACHINE_EXCEPTION);
         }
         return CompletableFuture.completedFuture(exceptionReply);
       }
@@ -853,13 +868,11 @@ class RaftServerImpl implements RaftServer.Division,
       // put the request into the pending queue
       pending = leaderState.addPendingRequest(permit, request, context);
       if (pending == null) {
-        cacheEntry.failWithException(new ResourceUnavailableException(
+        return cacheEntry.failWithException(new ResourceUnavailableException(
             getMemberId() + ": Failed to add a pending write request for " + request));
-        return cacheEntry.getReplyFuture();
       }
-      leaderState.notifySenders();
     }
-
+    leaderState.notifySenders();
     return pending.getFuture();
   }
 
@@ -897,7 +910,7 @@ class RaftServerImpl implements RaftServer.Division,
   }
 
   void stepDownOnJvmPause() {
-    role.getLeaderState().ifPresent(leader -> leader.submitStepDownEvent(LeaderState.StepDownReason.JVM_PAUSE));
+    role.getLeaderState().ifPresent(leader -> leader.submitStepDownEvent(StepDownReason.JVM_PAUSE));
   }
 
   private RaftClientRequest filterDataStreamRaftClientRequest(RaftClientRequest request)
@@ -1517,7 +1530,7 @@ class RaftServerImpl implements RaftServer.Division,
   static void logAppendEntries(boolean isHeartbeat, Supplier<String> message) {
     if (isHeartbeat) {
       if (LOG.isTraceEnabled()) {
-        LOG.trace("HEARTBEAT: " + message.get());
+        LOG.trace("HEARTBEAT: {}", message.get());
       }
     } else {
       if (LOG.isDebugEnabled()) {
@@ -1568,11 +1581,11 @@ class RaftServerImpl implements RaftServer.Division,
             AppendResult.NOT_LEADER, callId, RaftLog.INVALID_LOG_INDEX, isHeartbeat));
       }
       try {
-        future = changeToFollowerAndPersistMetadata(leaderTerm, true, "appendEntries");
+        future = changeToFollowerAndPersistMetadata(leaderTerm, true, Op.APPEND_ENTRIES);
       } catch (IOException e) {
         return JavaUtils.completeExceptionally(e);
       }
-      state.setLeader(leaderId, "appendEntries");
+      state.setLeader(leaderId, Op.APPEND_ENTRIES);
 
       if (!proto.getInitializing() && lifeCycle.compareAndTransition(State.STARTING, State.RUNNING)) {
         role.startFollowerState(this, Op.APPEND_ENTRIES);
@@ -1639,8 +1652,9 @@ class RaftServerImpl implements RaftServer.Division,
       return appendLogFuture.get();
     }
 
-    return appendLogFuture.updateAndGet(f -> f.thenCompose(
-            ignored -> JavaUtils.allOf(state.getLog().append(entries))))
+
+    return appendLogFuture.updateAndGet(f -> f.thenComposeAsync(
+            ignored -> JavaUtils.allOf(state.getLog().append(entries)), serverExecutor))
         .whenComplete((v, e) -> appendLogTermIndices.removeExisting(entriesTermIndices));
   }
 
