@@ -24,6 +24,7 @@ import org.apache.ratis.protocol.exceptions.StateMachineException;
 import org.apache.ratis.server.RaftConfiguration;
 import org.apache.ratis.server.RaftServerConfigKeys;
 import org.apache.ratis.server.impl.LeaderElection.Phase;
+import org.apache.ratis.server.protocol.RaftServerProtocol.Op;
 import org.apache.ratis.server.protocol.TermIndex;
 import org.apache.ratis.server.raftlog.LogProtoUtils;
 import org.apache.ratis.server.raftlog.RaftLog;
@@ -123,12 +124,7 @@ class ServerState {
     // On start the leader is null, start the clock now
     this.lastNoLeaderTime = new AtomicReference<>(Timestamp.currentTime());
     this.noLeaderTimeout = RaftServerConfigKeys.Notification.noLeaderTimeout(prop);
-
-    final LongSupplier getSnapshotIndexFromStateMachine = () -> Optional.ofNullable(stateMachine.getLatestSnapshot())
-        .map(SnapshotInfo::getIndex)
-        .filter(i -> i >= 0)
-        .orElse(RaftLog.INVALID_LOG_INDEX);
-    this.log = JavaUtils.memoize(() -> initRaftLog(getSnapshotIndexFromStateMachine, prop));
+    this.log = JavaUtils.memoize(() -> initRaftLog(() -> getSnapshotIndexFromStateMachine(stateMachine), prop));
     this.readRequests = new ReadRequests(prop, stateMachine);
     this.stateMachineUpdater = JavaUtils.memoize(() -> new StateMachineUpdater(
         stateMachine, server, this, getLog().getSnapshotIndex(), prop,
@@ -154,6 +150,16 @@ class ServerState {
     return memberId;
   }
 
+  private long getSnapshotIndexFromStateMachine(StateMachine stateMachine) {
+    final SnapshotInfo latest = stateMachine.getLatestSnapshot();
+    LOG.info("{}: getLatestSnapshot({}) returns {}", getMemberId(), stateMachine, latest);
+    if (latest == null) {
+      return RaftLog.INVALID_LOG_INDEX;
+    }
+    final long index = latest.getIndex();
+    return index >= 0 ? index : RaftLog.INVALID_LOG_INDEX;
+  }
+
   void writeRaftConfiguration(LogEntryProto conf) {
     getStorage().writeRaftConfiguration(conf);
   }
@@ -252,7 +258,7 @@ class ServerState {
    */
   void grantVote(RaftPeerId candidateId) {
     votedFor = candidateId;
-    setLeader(null, "grantVote");
+    setLeader(null, Op.REQUEST_VOTE);
   }
 
   void setLeader(RaftPeerId newLeaderId, Object op) {
@@ -265,7 +271,7 @@ class ServerState {
         suffix = "";
       } else {
         final Timestamp previous = lastNoLeaderTime.getAndSet(null);
-        suffix = ", leader elected after " + previous.elapsedTimeMs() + "ms";
+        suffix = ", leader elected after " + (previous != null ? previous.elapsedTimeMs() : 0) + "ms";
         server.setFirstElection(op);
         server.getStateMachine().event().notifyLeaderChanged(getMemberId(), newLeaderId);
       }
@@ -426,7 +432,7 @@ class ServerState {
       if (e instanceof InterruptedException) {
           Thread.currentThread().interrupt();
       }
-      LOG.warn(getMemberId() + ": Failed to join " + getStateMachineUpdater(), e);
+      LOG.warn("{}: Failed to join {}", getMemberId(), getStateMachineUpdater(), e);
     }
 
     try {
@@ -434,7 +440,7 @@ class ServerState {
         getLog().close();
       }
     } catch (Throwable e) {
-      LOG.warn(getMemberId() + ": Failed to close raft log " + getLog(), e);
+      LOG.warn("{}: Failed to close raft log {}", getMemberId(), getLog(), e);
     }
 
     try {
@@ -442,7 +448,7 @@ class ServerState {
         getStorage().close();
       }
     } catch (Throwable e) {
-      LOG.warn(getMemberId() + ": Failed to close raft storage " + getStorage(), e);
+      LOG.warn("{}: Failed to close raft storage {}", getMemberId(), getStorage(), e);
     }
   }
 
