@@ -244,7 +244,11 @@ public final class LogSegment {
         }
       });
       loadingTimes.incrementAndGet();
-      return Objects.requireNonNull(toReturn.get());
+      final LogEntryProto proto = toReturn.get();
+      if (proto == null) {
+        throw new RaftLogIOException("Failed to load log entry " + key);
+      }
+      return proto;
     }
   }
 
@@ -324,10 +328,10 @@ public final class LogSegment {
     }
 
     final LogRecord record = new LogRecord(totalFileSize, entry);
-    records.add(record);
     if (keepEntryInCache) {
       putEntryCache(record.getTermIndex(), entry, op);
     }
+    records.add(record);
     totalFileSize += getEntrySize(entry, op);
     endIndex = entry.getIndex();
   }
@@ -346,8 +350,10 @@ public final class LogSegment {
     }
     try {
       return cacheLoader.load(record);
+    } catch (RaftLogIOException e) {
+      throw e;
     } catch (Exception e) {
-      throw new RaftLogIOException(e);
+      throw new RaftLogIOException("Failed to loadCache for log entry " + record, e);
     }
   }
 
