@@ -26,6 +26,7 @@ import org.apache.ratis.protocol.RaftPeerId;
 import org.apache.ratis.server.RaftServer;
 import org.apache.ratis.server.RaftServerConfigKeys;
 import org.apache.ratis.server.impl.MiniRaftCluster;
+import org.apache.ratis.server.impl.PeerChanges;
 import org.apache.ratis.server.impl.RaftServerTestUtil;
 import org.apache.ratis.server.protocol.TermIndex;
 import org.apache.ratis.server.raftlog.RaftLog;
@@ -49,7 +50,6 @@ import java.io.File;
 import java.io.IOException;
 import java.nio.file.Files;
 import java.nio.file.Path;
-import java.util.Arrays;
 import java.util.Collections;
 import java.util.List;
 import java.util.concurrent.CompletableFuture;
@@ -79,6 +79,7 @@ public abstract class InstallSnapshotNotificationTests<CLUSTER extends MiniRaftC
 
     RaftServerConfigKeys.Log.setPurgeGap(prop, PURGE_GAP);
     RaftServerConfigKeys.Log.setSegmentSizeMax(prop, SizeInBytes.valueOf(1024)); // 1k segment
+    RaftServerConfigKeys.LeaderElection.setMemberMajorityAdd(prop, true);
   }
 
   private static final int SNAPSHOT_TRIGGER_THRESHOLD = 64;
@@ -239,15 +240,12 @@ public abstract class InstallSnapshotNotificationTests<CLUSTER extends MiniRaftC
       final boolean set = LEADER_SNAPSHOT_INFO_REF.compareAndSet(null, leaderSnapshotInfo);
       Assertions.assertTrue(set);
 
-      // add two more peers
-      final MiniRaftCluster.PeerChanges change = cluster.addNewPeers(2, true,
-          true);
+      // Add new peer(s)
+      final PeerChanges change = cluster.addNewPeers(1, true);
       // trigger setConfiguration
-      RaftServerTestUtil.runWithMinorityPeers(cluster, Arrays.asList(change.allPeersInNewConf),
-          peers -> cluster.setConfiguration(peers.toArray(RaftPeer.emptyArray())));
+      RaftServerTestUtil.runWithMinorityPeers(cluster, change.getPeersInNewConf(), cluster::setConfiguration);
 
-      RaftServerTestUtil
-          .waitAndCheckNewConf(cluster, change.allPeersInNewConf, 0, null);
+      RaftServerTestUtil.waitAndCheckNewConf(cluster, change.getPeersInNewConf(), 0, null);
 
       // Check the installed snapshot index on each Follower matches with the
       // leader snapshot.
@@ -389,14 +387,12 @@ public abstract class InstallSnapshotNotificationTests<CLUSTER extends MiniRaftC
             follower.getRaftLog().getNextIndex());
       }
 
-      // Add two more peers who will need snapshots from the leader.
-      final MiniRaftCluster.PeerChanges change = cluster.addNewPeers(2, true,
-          true);
+      // Add new peer(s) who will need snapshots from the leader.
+      final int numNewPeers = 1;
+      final PeerChanges change = cluster.addNewPeers(numNewPeers, true);
       // trigger setConfiguration
-      RaftServerTestUtil.runWithMinorityPeers(cluster, Arrays.asList(change.allPeersInNewConf),
-          peers -> cluster.setConfiguration(peers.toArray(RaftPeer.emptyArray())));
-      RaftServerTestUtil
-          .waitAndCheckNewConf(cluster, change.allPeersInNewConf, 0, null);
+      RaftServerTestUtil.runWithMinorityPeers(cluster, change.getPeersInNewConf(), cluster::setConfiguration);
+      RaftServerTestUtil.waitAndCheckNewConf(cluster, change.getPeersInNewConf(), 0, null);
 
       // Generate more data.
       try (final RaftClient client = cluster.createClient(leader.getId())) {
@@ -412,7 +408,7 @@ public abstract class InstallSnapshotNotificationTests<CLUSTER extends MiniRaftC
       }
 
       // Make sure each new peer got one snapshot notification.
-      Assertions.assertEquals(2, numSnapshotRequests.get());
+      Assertions.assertEquals(numNewPeers, numSnapshotRequests.get());
 
     } finally {
       cluster.shutdown();
@@ -479,13 +475,10 @@ public abstract class InstallSnapshotNotificationTests<CLUSTER extends MiniRaftC
       Assertions.assertTrue(set);
 
       // add one new peer
-      final MiniRaftCluster.PeerChanges change = cluster.addNewPeers(1, true, true);
+      final PeerChanges change = cluster.addNewPeers(1, true);
       // trigger setConfiguration
-      RaftServerTestUtil.runWithMinorityPeers(cluster, Arrays.asList(change.allPeersInNewConf),
-          peers -> cluster.setConfiguration(peers.toArray(RaftPeer.emptyArray())));
-
-      RaftServerTestUtil
-          .waitAndCheckNewConf(cluster, change.allPeersInNewConf, 0, null);
+      RaftServerTestUtil.runWithMinorityPeers(cluster, change.getPeersInNewConf(), cluster::setConfiguration);
+      RaftServerTestUtil.waitAndCheckNewConf(cluster, change.getPeersInNewConf(), 0, null);
 
       // Check the installed snapshot index on each Follower matches with the
       // leader snapshot.
@@ -556,14 +549,13 @@ public abstract class InstallSnapshotNotificationTests<CLUSTER extends MiniRaftC
       final boolean set = LEADER_SNAPSHOT_INFO_REF.compareAndSet(null, leaderSnapshotInfo);
       Assertions.assertTrue(set);
 
-      // add two more peers
-      final MiniRaftCluster.PeerChanges change = cluster.addNewPeers(2, true,
-          true);
+      // Add new peer(s)
+      final int numNewPeers = 1;
+      final PeerChanges change = cluster.addNewPeers(numNewPeers, true);
       // trigger setConfiguration
-      RaftServerTestUtil.runWithMinorityPeers(cluster, Arrays.asList(change.allPeersInNewConf),
-          peers -> cluster.setConfiguration(peers.toArray(RaftPeer.emptyArray())));
+      RaftServerTestUtil.runWithMinorityPeers(cluster, change.getPeersInNewConf(), cluster::setConfiguration);
 
-      RaftServerTestUtil.waitAndCheckNewConf(cluster, change.allPeersInNewConf, 0, null);
+      RaftServerTestUtil.waitAndCheckNewConf(cluster, change.getPeersInNewConf(), 0, null);
 
       // Check the installed snapshot index on each Follower matches with the
       // leader snapshot.
@@ -573,7 +565,7 @@ public abstract class InstallSnapshotNotificationTests<CLUSTER extends MiniRaftC
       }
 
       // Make sure each new peer got at least one snapshot notification.
-      Assertions.assertTrue(2 <= numSnapshotRequests.get());
+      Assertions.assertTrue(numNewPeers <= numSnapshotRequests.get());
     } finally {
       cluster.shutdown();
     }
