@@ -22,7 +22,6 @@ import org.apache.ratis.RaftTestUtil;
 import org.apache.ratis.client.RaftClient;
 import org.apache.ratis.conf.Parameters;
 import org.apache.ratis.conf.RaftProperties;
-import org.apache.ratis.proto.RaftProtos;
 import org.apache.ratis.protocol.ClientId;
 import org.apache.ratis.protocol.Message;
 import org.apache.ratis.protocol.RaftClientReply;
@@ -82,6 +81,8 @@ import java.util.stream.Collectors;
 import java.util.stream.Stream;
 import java.util.stream.StreamSupport;
 
+import static org.apache.ratis.proto.RaftProtos.RaftPeerRole.LISTENER;
+
 public abstract class MiniRaftCluster implements Closeable {
   public static final Logger LOG = LoggerFactory.getLogger(MiniRaftCluster.class);
 
@@ -238,18 +239,6 @@ public abstract class MiniRaftCluster implements Closeable {
     }
   }
 
-  public static class PeerChanges {
-    public final RaftPeer[] allPeersInNewConf;
-    public final RaftPeer[] newPeers;
-    public final RaftPeer[] removedPeers;
-
-    public PeerChanges(RaftPeer[] all, RaftPeer[] newPeers, RaftPeer[] removed) {
-      this.allPeersInNewConf = all;
-      this.newPeers = newPeers;
-      this.removedPeers = removed;
-    }
-  }
-
   public static RaftGroup initRaftGroup(Collection<String> ids, Collection<String> listenerIds) {
     Stream<RaftPeer> peer = ids.stream()
         .map(id -> RaftPeer.newBuilder().setId(id))
@@ -258,7 +247,7 @@ public abstract class MiniRaftCluster implements Closeable {
     Stream<RaftPeer> listener = listenerIds.stream()
         .map(id -> RaftPeer.newBuilder().setId(id))
         .map(MiniRaftCluster::assignAddresses)
-        .map(p -> p.setStartupRole(RaftProtos.RaftPeerRole.LISTENER))
+        .map(p -> p.setStartupRole(LISTENER))
         .map(RaftPeer.Builder::build);
     final RaftPeer[] peers = Stream.concat(peer, listener).toArray(RaftPeer[]::new);
 
@@ -351,9 +340,11 @@ public abstract class MiniRaftCluster implements Closeable {
         : JavaUtils.runRepeatedly(() -> LOG.info("TIMED-PRINT: {}.", printServers()), 10, 10, TimeUnit.SECONDS));
   }
 
-  /**
-   * start a stopped server again.
-   */
+  public void removeServer(RaftPeerId serverId) {
+    servers.remove(serverId);
+  }
+
+  /** Restart the server with the given id. */
   public RaftServer.Division restartServer(RaftPeerId serverId, boolean format) throws IOException {
     return restartServer(serverId, group, format);
   }
@@ -361,7 +352,7 @@ public abstract class MiniRaftCluster implements Closeable {
   public RaftServer.Division restartServer(RaftPeerId serverId, RaftGroup raftGroup, boolean format)
       throws IOException {
     killServer(serverId);
-    servers.remove(serverId);
+    removeServer(serverId);
 
     final RaftServer proxy = putNewServer(serverId, raftGroup, format);
     proxy.start();
@@ -443,43 +434,11 @@ public abstract class MiniRaftCluster implements Closeable {
 
   public PeerChanges addNewPeers(int number, boolean startNewPeer)
       throws IOException {
-    return addNewPeers(generateIds(number, servers.size()), startNewPeer, false);
-  }
-
-  public PeerChanges addNewPeers(int number, boolean startNewPeer,
-      boolean emptyPeer) throws IOException {
-    return addNewPeers(generateIds(number, servers.size()), startNewPeer, emptyPeer,
-        RaftProtos.RaftPeerRole.FOLLOWER);
-  }
-
-  public PeerChanges addNewPeers(String[] ids, boolean startNewPeer,
-      boolean emptyPeer) throws IOException {
-    return addNewPeers(ids, startNewPeer, emptyPeer, RaftProtos.RaftPeerRole.FOLLOWER);
-  }
-
-  public PeerChanges addNewPeers(int number, boolean startNewPeer,
-      boolean emptyPeer, RaftProtos.RaftPeerRole startRole) throws IOException {
-    return addNewPeers(generateIds(number, servers.size()), startNewPeer, emptyPeer, startRole);
-  }
-
-  public PeerChanges addNewPeers(String[] ids, boolean startNewPeer,
-      boolean emptyPeer, RaftProtos.RaftPeerRole startRole) throws IOException {
+    final String[] ids = generateIds(number, servers.size());
     LOG.info("Add new peers {}", Arrays.asList(ids));
 
     final Iterable<RaftPeerId> peerIds = CollectionUtils.as(Arrays.asList(ids), RaftPeerId::valueOf);
-    final RaftGroup raftGroup;
-    if (emptyPeer) {
-      raftGroup = RaftGroup.valueOf(group.getGroupId(), Collections.emptyList());
-    } else {
-      final Collection<RaftPeer> newPeers = StreamSupport.stream(peerIds.spliterator(), false)
-          .map(id -> RaftPeer.newBuilder().setId(id)
-              .setStartupRole(startRole))
-          .map(MiniRaftCluster::assignAddresses)
-          .map(RaftPeer.Builder::build)
-          .collect(Collectors.toSet());
-      newPeers.addAll(group.getPeers());
-      raftGroup = RaftGroup.valueOf(group.getGroupId(), newPeers);
-    }
+    final RaftGroup raftGroup = RaftGroup.valueOf(group.getGroupId(), Collections.emptyList());
 
     // create and add new RaftServers
     final Collection<RaftServer> newServers = putNewServers(peerIds, true, raftGroup);
@@ -491,12 +450,12 @@ public abstract class MiniRaftCluster implements Closeable {
       }
     }
 
-    final Collection<RaftPeer> newPeers = toRaftPeers(newServers);
-    final RaftPeer[] np = newPeers.toArray(RaftPeer.emptyArray());
-    newPeers.addAll(group.getPeers());
-    RaftPeer[] p = newPeers.toArray(RaftPeer.emptyArray());
-    group = RaftGroup.valueOf(group.getGroupId(), p);
-    return new PeerChanges(p, np, RaftPeer.emptyArray());
+    final List<RaftPeer> newPeers = toRaftPeers(newServers);
+    final List<RaftPeer> allPeers = new ArrayList<>(newPeers.size() + group.getPeers().size());
+    allPeers.addAll(newPeers);
+    allPeers.addAll(group.getPeers());
+    group = RaftGroup.valueOf(group.getGroupId(), allPeers);
+    return new PeerChanges(allPeers, newPeers, Collections.emptyList());
   }
 
   void startServers(Iterable<? extends RaftServer> raftServers) throws IOException {
@@ -511,7 +470,7 @@ public abstract class MiniRaftCluster implements Closeable {
    */
   public PeerChanges removePeers(int number, boolean removeLeader,
       Collection<RaftPeer> excluded) throws InterruptedException {
-    Collection<RaftPeer> raftPeers = new ArrayList<>(group.getPeers());
+    final List<RaftPeer> raftPeers = new ArrayList<>(group.getPeers());
     List<RaftPeer> removedPeers = new ArrayList<>(number);
     if (removeLeader) {
       final RaftPeer leader = RaftTestUtil.waitForLeader(this).getPeer();
@@ -529,9 +488,8 @@ public abstract class MiniRaftCluster implements Closeable {
         removed++;
       }
     }
-    final RaftPeer[] p = raftPeers.toArray(RaftPeer.emptyArray());
-    group = RaftGroup.valueOf(group.getGroupId(), p);
-    return new PeerChanges(p, RaftPeer.emptyArray(), removedPeers.toArray(RaftPeer.emptyArray()));
+    group = RaftGroup.valueOf(group.getGroupId(), raftPeers);
+    return new PeerChanges(raftPeers, Collections.emptyList(), removedPeers);
   }
 
   public void killServer(RaftPeerId id) {
@@ -813,15 +771,14 @@ public abstract class MiniRaftCluster implements Closeable {
   }
 
   public SetConfigurationRequest newSetConfigurationRequest(
-      ClientId clientId, RaftPeerId leaderId,
-      RaftPeer... raftPeers) {
+      ClientId clientId, RaftPeerId leaderId, List<RaftPeer> raftPeers) {
     return new SetConfigurationRequest(clientId, leaderId, getGroupId(), CallId.getDefault(),
         SetConfigurationRequest.Arguments.newBuilder().setServersInNewConf(raftPeers).build());
   }
 
-  public void setConfiguration(RaftPeer... raftPeers) throws IOException {
+  public void setConfiguration(List<RaftPeer> raftPeers) throws IOException {
     try(RaftClient client = createClient()) {
-      LOG.info("Start changing the configuration: {}", Arrays.asList(raftPeers));
+      LOG.info("Start changing the configuration: {}", raftPeers);
       final RaftClientReply reply = client.admin().setConfiguration(raftPeers);
       Preconditions.assertTrue(reply.isSuccess());
     }
