@@ -38,6 +38,7 @@ import org.apache.ratis.protocol.RaftClientReply;
 import org.apache.ratis.protocol.RaftPeerId;
 import org.apache.ratis.server.RaftServer;
 import org.apache.ratis.server.RaftServerConfigKeys;
+import org.apache.ratis.server.impl.PeerChanges;
 import org.apache.ratis.server.impl.RaftServerTestUtil;
 import org.apache.ratis.server.metrics.RaftServerMetricsImpl;
 import org.apache.ratis.server.raftlog.RaftLog;
@@ -49,16 +50,12 @@ import org.apache.ratis.util.FileUtils;
 import org.apache.ratis.util.JavaUtils;
 import org.apache.ratis.util.LifeCycle;
 import org.apache.ratis.util.Slf4jUtils;
-import org.junit.jupiter.api.AfterEach;
 import org.junit.jupiter.api.Assertions;
-import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import java.io.File;
-import java.io.IOException;
-import java.util.Arrays;
 import java.util.List;
 import java.util.Optional;
 import java.util.stream.Collectors;
@@ -67,11 +64,18 @@ import java.util.stream.LongStream;
 import org.apache.ratis.thirdparty.com.codahale.metrics.Timer;
 import org.slf4j.event.Level;
 
-public abstract class RaftSnapshotBaseTest extends BaseTest {
+public abstract class RaftSnapshotBaseTest<CLUSTER extends MiniRaftCluster>
+    extends BaseTest
+    implements MiniRaftCluster.Factory.Get<CLUSTER> {
   {
     Slf4jUtils.setLogLevel(RaftServer.Division.LOG, Level.DEBUG);
     Slf4jUtils.setLogLevel(RaftLog.LOG, Level.DEBUG);
-    Slf4jUtils.setLogLevel(RaftClient.LOG, Level.DEBUG);
+
+    final RaftProperties p = getProperties();
+    p.setClass(MiniRaftCluster.STATEMACHINE_CLASS_KEY, SimpleStateMachine4Testing.class, StateMachine.class);
+    RaftServerConfigKeys.Snapshot.setAutoTriggerThreshold(p, SNAPSHOT_TRIGGER_THRESHOLD);
+    RaftServerConfigKeys.Snapshot.setAutoTriggerEnabled(p, true);
+    RaftServerConfigKeys.LeaderElection.setMemberMajorityAdd(p, true);
   }
 
   static final Logger LOG = LoggerFactory.getLogger(RaftSnapshotBaseTest.class);
@@ -92,16 +96,16 @@ public abstract class RaftSnapshotBaseTest extends BaseTest {
     assertLogContent(leader, true);
   }
 
-  public static void assertLogContent(RaftServer.Division server, boolean isLeader) throws Exception {
+  public static void checkMetadataEntry(RaftServer.Division server) throws Exception {
     final RaftLog log = server.getRaftLog();
     final long lastIndex = log.getLastEntryTermIndex().getIndex();
     final LogEntryProto e = log.get(lastIndex);
     Assertions.assertTrue(e.hasMetadataEntry());
+    Assertions.assertEquals(log.getLastCommittedIndex() - 1, e.getMetadataEntry().getCommitIndex());
+  }
 
-    JavaUtils.attemptRepeatedly(() -> {
-      Assertions.assertEquals(log.getLastCommittedIndex() - 1, e.getMetadataEntry().getCommitIndex());
-      return null;
-    }, 50, BaseTest.HUNDRED_MILLIS, "CheckMetadataEntry", LOG);
+  public static void assertLogContent(RaftServer.Division server, boolean isLeader) throws Exception {
+    JavaUtils.attempt(() -> checkMetadataEntry(server), 50, HUNDRED_MILLIS, "checkMetadataEntry", LOG);
 
     SimpleStateMachine4Testing simpleStateMachine = SimpleStateMachine4Testing.get(server);
     if (isLeader) {
@@ -119,29 +123,6 @@ public abstract class RaftSnapshotBaseTest extends BaseTest {
     }
   }
 
-  private MiniRaftCluster cluster;
-
-  public abstract MiniRaftCluster.Factory<?> getFactory();
-
-  @BeforeEach
-  public void setup() throws IOException {
-    final RaftProperties prop = new RaftProperties();
-    prop.setClass(MiniRaftCluster.STATEMACHINE_CLASS_KEY,
-        SimpleStateMachine4Testing.class, StateMachine.class);
-    RaftServerConfigKeys.Snapshot.setAutoTriggerThreshold(
-        prop, SNAPSHOT_TRIGGER_THRESHOLD);
-    RaftServerConfigKeys.Snapshot.setAutoTriggerEnabled(prop, true);
-    this.cluster = getFactory().newCluster(1, prop);
-    cluster.start();
-  }
-
-  @AfterEach
-  public void tearDown() {
-    if (cluster != null) {
-      cluster.shutdown();
-    }
-  }
-
   /**
    * Keep generating writing traffic and make sure snapshots are taken.
    * We then restart the whole raft peer and check if it can correctly load
@@ -149,8 +130,13 @@ public abstract class RaftSnapshotBaseTest extends BaseTest {
    */
   @Test
   public void testRestartPeer() throws Exception {
-    RaftTestUtil.waitForLeader(cluster);
-    final RaftPeerId leaderId = cluster.getLeader().getId();
+    runWithNewCluster(1, this::runTestRestartPeer);
+
+  }
+
+  void runTestRestartPeer(CLUSTER cluster) throws Exception {
+    LOG.info("runTestRestartPeer");
+    final RaftPeerId leaderId = RaftTestUtil.waitForLeader(cluster).getId();
     int i = 0;
     try(final RaftClient client = cluster.createClient(leaderId)) {
       for (; i < SNAPSHOT_TRIGGER_THRESHOLD * 2 - 1; i++) {
@@ -180,7 +166,7 @@ public abstract class RaftSnapshotBaseTest extends BaseTest {
 
   public static boolean exists(File f) {
     if (f.exists()) {
-      LOG.info("File exists: " + f);
+      LOG.info("File exists: {}", f);
       return true;
     }
     return false;
@@ -193,11 +179,15 @@ public abstract class RaftSnapshotBaseTest extends BaseTest {
    */
   @Test
   public void testBasicInstallSnapshot() throws Exception {
+    runWithNewCluster(1, this::runTestBasicInstallSnapshot);
+  }
+
+  void runTestBasicInstallSnapshot(CLUSTER cluster) throws Exception {
+    LOG.info("runTestBasicInstallSnapshot");
     final List<LogSegmentPath> logs;
     int i = 0;
     try {
-      RaftTestUtil.waitForLeader(cluster);
-      final RaftPeerId leaderId = cluster.getLeader().getId();
+      final RaftPeerId leaderId = RaftTestUtil.waitForLeader(cluster).getId();
 
       try(final RaftClient client = cluster.createClient(leaderId)) {
         for (; i < SNAPSHOT_TRIGGER_THRESHOLD * 2 - 1; i++) {
@@ -236,23 +226,20 @@ public abstract class RaftSnapshotBaseTest extends BaseTest {
         Assertions.assertTrue(client.io().send(new SimpleMessage("m" + i)).isSuccess());
       }
 
-      // add two more peers
-      String[] newPeers = new String[]{"s3", "s4"};
-      MiniRaftCluster.PeerChanges change = cluster.addNewPeers(
-          newPeers, true, false);
+      // add a new peer
+      final PeerChanges change = cluster.addNewPeers(1, true);
       // trigger setConfiguration
-      RaftServerTestUtil.runWithMinorityPeers(cluster, Arrays.asList(change.allPeersInNewConf),
-          peers -> cluster.setConfiguration(peers.toArray(RaftPeer.emptyArray())));
+      RaftServerTestUtil.runWithMinorityPeers(cluster, change.getPeersInNewConf(), cluster::setConfiguration);
 
-      for (String newPeer : newPeers) {
-        final RaftServer.Division s = cluster.getDivision(RaftPeerId.valueOf(newPeer));
+      for (RaftPeer newPeer : change.getAddedPeers()) {
+        final RaftServer.Division s = cluster.getDivision(newPeer.getId());
         SimpleStateMachine4Testing simpleStateMachine = SimpleStateMachine4Testing.get(s);
         Assertions.assertSame(LifeCycle.State.RUNNING, simpleStateMachine.getLifeCycleState());
       }
 
       // Verify installSnapshot counter on leader before restart.
       verifyInstallSnapshotMetric(cluster.getLeader());
-      RaftServerTestUtil.waitAndCheckNewConf(cluster, change.allPeersInNewConf, 0, null);
+      RaftServerTestUtil.waitAndCheckNewConf(cluster, change.getPeersInNewConf(), 0, null);
 
       Timer timer = getTakeSnapshotTimer(cluster.getLeader());
       long count = timer.getCount();
@@ -275,6 +262,11 @@ public abstract class RaftSnapshotBaseTest extends BaseTest {
    */
   @Test
   public void testInstallSnapshotDuringBootstrap() throws Exception {
+    runWithNewCluster(1, this::runTestInstallSnapshotDuringBootstrap);
+  }
+
+  void runTestInstallSnapshotDuringBootstrap(CLUSTER cluster) throws Exception {
+    LOG.info("runTestInstallSnapshotDuringBootstrap");
     int i = 0;
     try {
       RaftTestUtil.waitForLeader(cluster);
@@ -299,23 +291,20 @@ public abstract class RaftSnapshotBaseTest extends BaseTest {
 
       assertLeaderContent(cluster);
 
-      // add two more peers
-      String[] newPeers = new String[]{"s3", "s4"};
-      MiniRaftCluster.PeerChanges change = cluster.addNewPeers(
-          newPeers, true, false);
+      // add a new peer
+      final PeerChanges change = cluster.addNewPeers(1, true);
       // trigger setConfiguration
-      RaftServerTestUtil.runWithMinorityPeers(cluster, Arrays.asList(change.allPeersInNewConf),
-          peers -> cluster.setConfiguration(peers.toArray(RaftPeer.emptyArray())));
+      RaftServerTestUtil.runWithMinorityPeers(cluster, change.getPeersInNewConf(), cluster::setConfiguration);
 
-      for (String newPeer : newPeers) {
-        final RaftServer.Division s = cluster.getDivision(RaftPeerId.valueOf(newPeer));
+      for (RaftPeer newPeer : change.getAddedPeers()) {
+        final RaftServer.Division s = cluster.getDivision(newPeer.getId());
         SimpleStateMachine4Testing simpleStateMachine = SimpleStateMachine4Testing.get(s);
         Assertions.assertSame(LifeCycle.State.RUNNING, simpleStateMachine.getLifeCycleState());
       }
 
       // Verify installSnapshot counter on leader
       verifyInstallSnapshotMetric(cluster.getLeader());
-      RaftServerTestUtil.waitAndCheckNewConf(cluster, change.allPeersInNewConf, 0, null);
+      RaftServerTestUtil.waitAndCheckNewConf(cluster, change.getPeersInNewConf(), 0, null);
     } finally {
       cluster.shutdown();
     }
