@@ -19,7 +19,7 @@ package org.apache.ratis.netty;
 
 import org.apache.ratis.server.impl.BlockRequestHandlingInjection;
 import org.apache.ratis.server.impl.LeaderElectionTests;
-import org.junit.Test;
+import org.junit.jupiter.api.Test;
 
 public class TestLeaderElectionWithNetty
     extends LeaderElectionTests<MiniRaftClusterWithNetty>
@@ -29,7 +29,7 @@ public class TestLeaderElectionWithNetty
   public void testEnforceLeader() throws Exception {
     super.testEnforceLeader();
 
-    MiniRaftClusterWithNetty.sendServerRequest.clear();
+    MiniRaftClusterWithNetty.SEND_SERVER_REQUEST.clear();
     BlockRequestHandlingInjection.getInstance().unblockAll();
   }
 
